@@ -78,6 +78,7 @@
 #[allow(dead_code)]
 pub const USER_INTERFACE_GUIDELINES: () = ();
 
+use std::borrow::Cow;
 use std::fmt::Write;
 use std::path::PathBuf;
 use std::time::Duration;
@@ -86,8 +87,6 @@ use std::time::Duration;
 use clap::{Command, CommandFactory, Parser, Subcommand};
 use clap::builder::StyledStr;
 
-pub mod autocrypt;
-
 use sequoia_openpgp as openpgp;
 use openpgp::Fingerprint;
 
@@ -95,19 +94,44 @@ use openpgp::Fingerprint;
 pub mod examples;
 
 pub mod cert;
+pub mod config;
 pub mod decrypt;
+pub mod download;
 pub mod encrypt;
 pub mod inspect;
 pub mod key;
+pub mod keyring;
 pub mod network;
 pub mod output;
+pub mod packet;
 pub mod pki;
 pub mod sign;
-pub mod toolbox;
 pub mod verify;
 pub mod version;
 
 pub mod types;
+use types::paths::{StateDirectory, StateDirectoryValueParser};
+use types::version::Version;
+use types::version::VersionInvalidPositionValueParser;
+
+/// Escapes `s` so when used on the command line, it is recognized as
+/// a single token with the same value.
+///
+/// "{:?}" escapes a string, but it always adds quotes, which can be
+/// ugly.  We first check check if adding quotes is necessary.
+/// Specifically, if `s` just consists of alphanumeric characters and
+/// a few others, then we don't need quotes.
+pub fn escape_for_shell(s: &str) -> Cow<str> {
+    if s.chars().all(|c| {
+        c.is_ascii_alphanumeric()
+            || ['@', '.', ',', '+', '-', '_', '/'].contains(&c)
+    })
+    {
+        Cow::Borrowed(s)
+    } else {
+        Cow::Owned(format!("{:?}", s))
+    }
+}
 
 /// The seconds in a day
 pub const SECONDS_IN_DAY : u64 = 24 * 60 * 60;
@@ -121,13 +145,19 @@ pub const KEY_VALIDITY_IN_YEARS: u64 = 3;
 pub const KEY_VALIDITY_DURATION: Duration =
     Duration::new(SECONDS_IN_YEAR * KEY_VALIDITY_IN_YEARS, 0);
 /// The default validity (in years) for third party certifications
-pub const THIRD_PARTY_CERTIFICATION_VALIDITY_IN_YEARS: u64 = 5;
+pub const THIRD_PARTY_CERTIFICATION_VALIDITY_IN_YEARS: u64 = 10;
 /// The default validity period (as Duration) for third party certifications
 pub const THIRD_PARTY_CERTIFICATION_VALIDITY_DURATION: Duration = Duration::new(
     SECONDS_IN_YEAR * THIRD_PARTY_CERTIFICATION_VALIDITY_IN_YEARS,
     0,
 );
 
+/// The default time (in days) to retire a certificate after rotation.
+pub const KEY_ROTATE_RETIRE_IN_IN_DAYS: u64 = 182;
+/// The default time to retire a certificate after rotation.
+pub const KEY_ROTATE_RETIRE_IN_DURATION: Duration =
+    Duration::new(SECONDS_IN_DAY * KEY_ROTATE_RETIRE_IN_IN_DAYS, 0);
+
 pub const GLOBAL_OPTIONS_HEADER: &str = "Global Options";
 
 /// Builds the top-level Clap command.
@@ -146,7 +176,10 @@ pub fn build(globals_hidden: bool) -> Command {
         .into_boxed_str(),
     ) as &str;
 
-    let mut command = SqCommand::command().version(sq_version);
+    let mut command = SqCommand::command()
+        .version(sq_version)
+    // To improve readability limit the width of the text columns.
+        .max_term_width(100);
 
     // Change the globals to be hidden.
     if globals_hidden {
@@ -172,7 +205,7 @@ pub fn build(globals_hidden: bool) -> Command {
                 };
 
             after_long_help.write_str(&format!("\
-{}:\nSee 'sq --help' for a description of the global options.",
+{}:\n  See 'sq --help' for a description of the global options.",
                                                GLOBAL_OPTIONS_HEADER))
                 .expect("Can write to string");
 
@@ -205,50 +238,95 @@ pub fn build(globals_hidden: bool) -> Command {
     about = "A command-line frontend for Sequoia, an implementation of OpenPGP",
     long_about = "A command-line frontend for Sequoia, an implementation of OpenPGP
 
-Functionality is grouped and available using subcommands.  This
-interface is not completely stateless.  In particular, the user's
-default certificate store is used.  This can be disabled using
-`--no-cert-store`.  Similarly, a key store is used to manage and
-protect secret key material.  This can be disabled using
-`--no-key-store`.
+Functionality is grouped and available using subcommands.  This \
+interface is not completely stateless.  In particular, the user's \
+default certificate store is used.  This can be disabled using \
+`--cert-store=none`.  Similarly, a key store is used to manage and \
+protect secret key material.  This can be disabled using \
+`--key-store=none`.
 
-OpenPGP data can be provided in binary or ASCII armored form.  This
-will be handled automatically.  Emitted OpenPGP data is ASCII armored
+OpenPGP data can be provided in binary or ASCII armored form.  This \
+will be handled automatically.  Emitted OpenPGP data is ASCII armored \
 by default.
 
-We use the term \"certificate\", or \"cert\" for short, to refer to OpenPGP
-keys that do not contain secrets.  Conversely, we use the term \"key\"
+We use the term \"certificate\", or \"cert\" for short, to refer to OpenPGP \
+keys that do not contain secrets.  Conversely, we use the term \"key\" \
 to refer to OpenPGP keys that do contain secrets.
 ",
     subcommand_required = true,
     arg_required_else_help = true,
     disable_colored_help = true,
     disable_version_flag = true,
+    // We want a top-level `help` subcommand, but we don't want
+    // subcommands groups (like `sq pki`) to have a `help` subcommand.
+    // Users get used to being able to use `help` instead of `--help`,
+    // and then are confused when `sq pki authenticate help` (i.e.,
+    // using the `help` subcommand on an action) doesn't work.
+    //
+    // Note: this option is recursive.  So if we disable it here, then
+    // we have to enable it for all of the top-level subcommands.
+    disable_help_subcommand = false,
 )]
 pub struct SqCommand {
     #[clap(
-        short = 'f',
-        long = "force",
+        long = "cli-version",
+        global = true,
+        value_parser = VersionInvalidPositionValueParser::default(),
+        help_heading = GLOBAL_OPTIONS_HEADER,
+        help = "Select a CLI version",
+        long_help = format!("\
+Select a CLI version
+
+`sq`'s CLI is versioned using a semantic versioning scheme.  Setting \
+this options causes `sq` to error out if it does not implement an \
+interface that is compatible with the specified version.  For \
+instance, if you set this to 1.1.0 and `sq` only implements version
+1.0.0 of the interface, then `sq` will error out.
+
+`sq` may implement multiple interfaces (e.g., 1.1.4, and 2.0.5).  By \
+default, it selects the newest version.  As such, if you require a \
+particular interface, you need to set this option for every call to \
+`sq`.
+
+This option must be the first option on the command line.
+
+This version of `sq` implements version {}.{}.{} of the CLI interface.
+
+Stable since 1.2.0.
+",
+                            env!("CARGO_PKG_VERSION_MAJOR"),
+                            env!("CARGO_PKG_VERSION_MINOR"),
+                            env!("CARGO_PKG_VERSION_PATCH")),
+    )]
+    pub cli_version: Option<Version>,
+
+    #[clap(
+        long = "overwrite",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Overwrite existing files",
     )]
-    pub force: bool,
+    pub overwrite: bool,
+
     #[clap(
-        long,
+        long = "home",
+        value_name = "PATH",
         env = "SEQUOIA_HOME",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Set the home directory.",
+        help = "Set the home directory",
         long_help = format!("\
-Set the home directory.
+Set the home directory
 
 Sequoia's default home directory is `{}`.  When using the default \
 location, files are placed according to the local standard, \
 e.g., the XDG Base Directory Specification.  When an alternate \
 location is specified, the user data, configuration files, and \
 cache data are placed under a single, unified directory.  This is \
-a lightweight way to partially isolate `sq`.",
+a lightweight way to partially isolate `sq`.
+
+Use 'default' to explicitly use the default location, use 'none' to \
+not use a home directory.",
             sequoia_directories::Home::default_location()
                 .map(|p| {
                     let p = p.display().to_string();
@@ -261,29 +339,19 @@ a lightweight way to partially isolate `sq`.",
                     p
                 })
                 .unwrap_or("<unknown>".to_string())),
+        value_parser = StateDirectoryValueParser::default(),
     )]
-    pub home: Option<PathBuf>,
-    #[clap(
-        long,
-        global = true,
-        help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Disable the use of the key store.",
-        long_help = "\
-Disable the use of the key store.
+    pub home: Option<StateDirectory>,
 
-It is still possible to use functionality that does not require the
-key store."
-    )]
-    pub no_key_store: bool,
     #[clap(
-        long,
+        long = "key-store",
         value_name = "PATH",
-        env = "SQ_KEY_STORE",
-        conflicts_with_all = &[ "no_key_store" ],
+        env = "SEQUOIA_KEY_STORE",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Override the key store server and its data",
-        long_help = format!("\
+        long_help = format!("Override the key store server and its data
+
 A key store server manages and protects secret key material.  By \
 default, `sq` connects to the key store server for Sequoia's default \
 home directory (see `--home`), {}.  If no key store server is running, \
@@ -291,7 +359,10 @@ one is started.
 
 This option causes `sq` to use an alternate key store server.  If \
 necessary, a key store server is started, and configured to look for \
-its data in the specified location.",
+its data in the specified location.
+
+Use 'default' to explicitly use the default server, use 'none' to \
+not use a key store.",
             sequoia_directories::Home::default()
                 .map(|home| {
                     let p = home.data_dir(sequoia_directories::Component::Keystore);
@@ -305,32 +376,27 @@ its data in the specified location.",
                     p
                 })
                 .unwrap_or("<unknown>".to_string())),
+        value_parser = StateDirectoryValueParser::default(),
     )]
-    pub key_store: Option<PathBuf>,
+    pub key_store: Option<StateDirectory>,
+
     #[clap(
-        long,
-        global = true,
-        help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Disable the use of a certificate store",
-        long_help = "\
-Disable the use of a certificate store.  Normally sq uses the user's \
-standard cert-d, which is located in `$HOME/.local/share/pgp.cert.d`."
-    )]
-    pub no_cert_store: bool,
-    #[clap(
-        long,
+        long = "cert-store",
         value_name = "PATH",
-        env = "SQ_CERT_STORE",
-        conflicts_with_all = &[ "no_cert_store" ],
+        env = "SEQUOIA_CERT_STORE",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Specify the location of the certificate store",
         long_help = format!("\
-Specify the location of the certificate store.  By default, `sq` uses \
+Specify the location of the certificate store
+
+By default, `sq` uses \
 the OpenPGP certificate directory in Sequoia's home directory (see `--home`), \
 {}.  This can be overridden by setting the `PGP_CERT_D` environment \
-variable.  That in turn can be overridden by setting the `SQ_CERT_STORE` \
-environment variable.",
+variable.
+
+Use 'default' to explicitly use the default cert store, use 'none' to \
+not use a cert store.",
             sequoia_directories::Home::default()
                 .map(|home| {
                     let p = home.data_dir(sequoia_directories::Component::CertD);
@@ -344,23 +410,10 @@ environment variable.",
                     p
                 })
                 .unwrap_or("<unknown>".to_string())),
+        value_parser = StateDirectoryValueParser::default(),
     )]
-    pub cert_store: Option<PathBuf>,
-    #[clap(
-        long,
-        value_name = "PATH",
-        env = "PEP_CERT_STORE",
-        global = true,
-        help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Specify the location of a pEp certificate store",
-        long_help = "\
-Specify the location of a pEp certificate store.  sq does not use a \
-pEp certificate store by default; it must be explicitly enabled \
-using this argument or the corresponding environment variable, \
-PEP_CERT_STORE.  The pEp Engine's default certificate store is at \
-`$HOME/.pEp/keys.db`."
-    )]
-    pub pep_cert_store: Option<PathBuf>,
+    pub cert_store: Option<StateDirectory>,
+
     #[clap(
         long,
         value_name = "PATH",
@@ -368,58 +421,43 @@ PEP_CERT_STORE.  The pEp Engine's default certificate store is at \
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Specify the location of a keyring to use",
         long_help = "\
-Specify the location of a keyring to use.  Keyrings are used in \
+Specify the location of a keyring to use
+
+Keyrings are used in \
 addition to any certificate store.  The content of the keyring is \
 not imported into the certificate store.  When a certificate is \
 looked up, it is looked up in all keyrings and any certificate \
 store, and the results are merged together."
     )]
     pub keyring: Vec<PathBuf>,
-    #[clap(
-        long = "output-format",
-        value_name = "FORMAT",
-        default_value = "human-readable",
-        env = "SQ_OUTPUT_FORMAT",
-        global = true,
-        help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Produce output in FORMAT, if possible",
-    )]
-    pub output_format: output::OutputFormat,
-    #[clap(
-        long = "output-version",
-        value_name = "VERSION",
-        env = "SQ_OUTPUT_VERSION",
-        global = true,
-        help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Produce output variant VERSION.",
-        long_help = "Produce output variant VERSION, such as 0.0.0. \
-                     The default is the newest version. The output version \
-                     is separate from the version of the sq program. To see \
-                     the current supported versions, use output-versions \
-                     subcommand."
-    )]
-    pub output_version: Option<String>,
+
     #[clap(
         long = "known-notation",
         value_name = "NOTATION",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Add NOTATION to the list of known notations",
-        long_help = "Add NOTATION to the list of known notations. \
-            This is used when validating signatures. \
-            Signatures that have unknown notations with the \
-            critical bit set are considered invalid."
+        long_help = "Add NOTATION to the list of known notations
+
+
+This is used when validating signatures. \
+Signatures that have unknown notations with the \
+critical bit set are considered invalid."
     )]
     // TODO is this the right type?
     pub known_notation: Vec<String>,
+
     #[clap(
         long = "time",
+        allow_hyphen_values = true,
         value_name = "TIME",
-        help = "Set the reference time as ISO 8601 formatted timestamp",
+        help = "Set the reference time as an ISO 8601 formatted timestamp",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
         long_help = "\
-Set the reference time as an ISO 8601 formatted timestamp.  Normally, \
+Set the reference time as an ISO 8601 formatted timestamp
+
+Normally, \
 commands use the current time as the reference time.  This argument allows \
 the user to use a difference reference time.  For instance, when creating a \
 key using `sq key generate`, the creation time is normally set to the \
@@ -439,15 +477,47 @@ $ sq --time 20130721T0550+0200 verify msg.pgp
 ",
     )]
     pub time: Option<types::Time>,
+    #[clap(
+        long = "policy-as-of",
+        allow_hyphen_values = true,
+        value_name = "TIME",
+        help = "Select the cryptographic policy as of the specified time",
+        global = true,
+        help_heading = GLOBAL_OPTIONS_HEADER,
+        long_help = "\
+Select the cryptographic policy as of the specified time
+
+The time is \
+expressed as an ISO 8601 formatted timestamp.  The policy determines \
+what cryptographic constructs are allowed.
+
+If you are working with a message that sq rejects, because it is \
+protected by cryptographic constructs that are now considered broken, \
+you can use this option to select a different cryptographic policy.  \
+If you are relying on the cryptography, e.g., you are verifying a \
+signature, then you should only do this if you are confident that the \
+message hasn't been tampered with.
+
+TIME is interpreted as an ISO 8601 timestamp.  To set the \
+policy time to January 1, 2007 at midnight UTC, you can do:
+
+$ sq --policy-as-of 20070101 verify msg.pgp
+
+Defaults to the reference time, which can be set using --time.
+",
+    )]
+    pub policy_as_of: Option<types::Time>,
+
     #[clap(
         long = "trust-root",
         value_name = "FINGERPRINT|KEYID",
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Consider the specified certificate to be a trust root",
-        long_help = "Consider the specified certificate to be a trust root. \
-                     Trust roots are used by trust models, e.g., the Web of \
-                     Trust, to authenticate certificates and User IDs."
+        long_help = "Consider the specified certificate to be a trust root
+
+Trust roots are used by trust models, e.g., the Web of \
+Trust, to authenticate certificates and User IDs."
     )]
     pub trust_roots: Vec<Fingerprint>,
     #[clap(
@@ -457,7 +527,7 @@ $ sq --time 20130721T0550+0200 verify msg.pgp
         help_heading = GLOBAL_OPTIONS_HEADER,
         help = "Seed the password cache with the specified password",
         long_help = "\
-Seed the password cache with the specified password.
+Seed the password cache with the specified password
 
 The password is added to the password cache.  When decrypting secret \
 key material, the password cache is only used if the key is not \
@@ -468,14 +538,46 @@ Note that the entire key file will be used as the password, including \
 any surrounding whitespace like a trailing newline.",
     )]
     pub password_file: Vec<PathBuf>,
+
     #[clap(
-        short,
+        short = 'v',
         long,
         global = true,
         help_heading = GLOBAL_OPTIONS_HEADER,
-        help = "Be more verbose.",
+        help = "Be more verbose",
+        long_help = config::augment_help(
+            "ui.verbose",
+            "Be more verbose"),
     )]
     pub verbose: bool,
+
+    #[clap(
+        short = 'q',
+        long = "quiet",
+        global = true,
+        help_heading = GLOBAL_OPTIONS_HEADER,
+        help = "Be more quiet",
+        long_help = config::augment_help(
+            "ui.quiet",
+            "Be more quiet"),
+        conflicts_with = "verbose",
+    )]
+    pub quiet: bool,
+
+    #[clap(
+        long = "batch",
+        global = true,
+        help_heading = GLOBAL_OPTIONS_HEADER,
+        help = "Prevents any kind of prompting",
+        long_help = "\
+Prevents any kind of prompting
+
+Enables batch mode.  In batch mode, sq will never ask for user input,
+such as prompting for passwords.
+",
+    )]
+    pub batch: bool,
+
     #[clap(subcommand)]
     pub subcommand: SqSubcommands,
 }
@@ -497,6 +599,7 @@ pub enum SqSubcommands {
 
     Sign(sign::Command),
     Verify(verify::Command),
+    Download(download::Command),
 
     Inspect(inspect::Command),
 
@@ -504,10 +607,11 @@ pub enum SqSubcommands {
     Key(key::Command),
     Pki(pki::Command),
 
-    Autocrypt(autocrypt::Command),
     Network(network::Command),
 
-    Toolbox(toolbox::Command),
+    Keyring(keyring::Command),
+    Packet(packet::Command),
 
+    Config(config::Command),
     Version(version::Command),
 }
