@@ -4,7 +4,7 @@ use clap::{Parser, Subcommand};
 
 pub mod dane;
 pub mod keyserver;
-pub mod fetch;
+pub mod search;
 pub mod wkd;
 
 #[derive(Parser, Debug)]
@@ -14,12 +14,13 @@ pub mod wkd;
     long_about =
 "Retrieve and publish certificates over the network
 
-OpenPGP certificates can be discovered and updated from, and published
-on services accessible over the network.  This is a collection of
+OpenPGP certificates can be discovered and updated from, and published \
+on services accessible over the network.  This is a collection of \
 commands to interact with these services.
 ",
     subcommand_required = true,
     arg_required_else_help = true,
+    disable_help_subcommand = true,
 )]
 pub struct Command {
     #[clap(subcommand)]
@@ -28,7 +29,7 @@ pub struct Command {
 
 #[derive(Debug, Subcommand)]
 pub enum Subcommands {
-    Fetch(fetch::Command),
+    Search(search::Command),
     Keyserver(keyserver::Command),
     Wkd(wkd::Command),
     Dane(dane::Command),
