@@ -1,7 +1,6 @@
 use std::cmp::Ordering;
 use std::io::IsTerminal;
 use std::io:: Write;
-use std::process::exit;
 use std::time::SystemTime;
 use std::sync::Arc;
 
@@ -14,7 +13,6 @@ use openpgp::Result;
 use openpgp::armor;
 use openpgp::cert::prelude::*;
 use openpgp::crypto::Signer;
-use openpgp::parse::Parse;
 use openpgp::packet::prelude::*;
 use openpgp::policy::Policy;
 use openpgp::policy::StandardPolicy;
@@ -32,10 +30,12 @@ use cert_store::StoreUpdate;
 use crate::{
     Sq,
     cli::cert::lint::Command,
+    cli::types::cert_designator::CertDesignator,
     commands::FileOrStdout,
+    common::ui,
+    sq::TrustThreshold,
 };
 
-
 fn update_cert_revocation(sq: &Sq,
                           cert: &Cert, rev: &Signature,
                           reference_time: &SystemTime)
@@ -192,9 +192,6 @@ fn update_subkey_binding<P>(sq: &Sq,
 }
 
 pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
-    // If there were any errors reading the input.
-    let mut bad_input = false;
-
     // Number of certs that have issues.
     let mut certs_with_issues = 0;
     // Whether we were unable to fix at least one issue.
@@ -202,12 +199,12 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
 
     // Standard policy that unconditionally rejects SHA-1: this is
     // where we want to be.
-    let mut sp = StandardPolicy::new();
+    let mut sp = StandardPolicy::at(sq.policy_as_of);
     sp.reject_hash(HashAlgorithm::SHA1);
     let sp = &sp;
 
     // A standard policy that also accepts SHA-1.
-    let mut sp_sha1 = StandardPolicy::new();
+    let mut sp_sha1 = StandardPolicy::at(sq.policy_as_of);
     sp_sha1.accept_hash(HashAlgorithm::SHA1);
     let sp_sha1 = &sp_sha1;
 
@@ -232,13 +229,19 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
     let mut certs_with_signing_subkeys = 0;
     let mut certs_with_sha1_protected_backsig = 0;
 
-    if args.list_keys {
-        args.quiet = true;
-    }
-
     let reference_time = sq.time;
 
-    let mut out = if args.output.is_some() || ! args.cert_file.is_empty() {
+    // If no inputs are given, read from stdin.
+    if args.certs.is_empty() {
+        args.certs.designators.push(CertDesignator::Stdin);
+    }
+
+    let certs = sq.resolve_certs_or_fail(
+        &args.certs, TrustThreshold::Full)?;
+
+    let mut out = if args.output.is_some()
+        || args.certs.iter().any(|d| d.from_file() || d.from_stdin())
+    {
         let output = if let Some(output) = args.output {
             output
         } else {
@@ -246,8 +249,8 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
         };
 
         Some(output.create_pgp_safe(
-            sq.force, args.binary,
-            if args.export_secret_keys {
+            &sq, false,
+            if certs.iter().any(|c| c.is_tsk()) {
                 armor::Kind::SecretKey
             } else {
                 armor::Kind::PublicKey
@@ -256,46 +259,8 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
         None
     };
 
-    'next_input: for input in args.cert.into_iter().map(Ok)
-        .chain(args.cert_file.into_iter().map(Err))
     {
-        let certs = match input {
-            Ok(kh) => {
-                let cert = sq.lookup_one(&kh, None, true)?;
-                vec![cert]
-            }
-            Err(input) => {
-                let filename = match input.inner() {
-                    Some(path) => path.display().to_string(),
-                    None => "/dev/stdin".to_string(),
-                };
-
-                let mut input_reader = input.open()?;
-                let mut certs = Vec::new();
-                let certp = CertParser::from_buffered_reader(&mut input_reader)?;
-                for (certi, certo) in certp.enumerate() {
-                    match certo {
-                        Err(err) => {
-                            if ! args.quiet {
-                                if certi == 0 {
-                                    wprintln!("{:?} does not appear to be a keyring: {}",
-                                              filename, err);
-                                } else {
-                                    wprintln!("Encountered an error parsing {:?}: {}",
-                                              filename, err);
-                                }
-                            }
-                            bad_input = true;
-                            continue 'next_input;
-                        }
-                        Ok(cert) => certs.push(cert)
-                    }
-                }
-                certs
-            }
-        };
-
-        'next_cert: for cert in certs.into_iter() {
+        'next_cert: for cert in certs {
             // Whether we found at least one issue.
             let mut found_issue = false;
 
@@ -309,13 +274,9 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                         if ! found_issue {
                             certs_with_issues += 1;
                             found_issue = true;
-                            if args.list_keys {
-                                println!("{}",
-                                         cert.fingerprint().to_hex());
-                            }
                         }
-                        if ! args.quiet {
-                            wprintln!($($arg)*);
+                        if ! sq.quiet() {
+                            weprintln!($($arg)*);
                         }
                     }
                 }};
@@ -328,11 +289,7 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                     if updates.len() > 0 {
                         let cert = cert.insert_packets(updates)?;
                         if let Some(mut out) = out.as_mut() {
-                            if args.export_secret_keys {
-                                cert.as_tsk().serialize(&mut out)?;
-                            } else {
-                                cert.serialize(&mut out)?;
-                            }
+                            cert.as_tsk().serialize(&mut out)?;
                         } else {
                             let fpr = cert.fingerprint();
                             let cert_store = sq.cert_store_or_else()?;
@@ -453,14 +410,14 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                                 }
                                 Err(err) => {
                                     unfixed_issue += 1;
-                                    wprintln!("Certificate {}: \
-                                               Failed to update \
-                                               revocation certificate \
-                                               {:02X}{:02X}: {}",
-                                              cert.keyid().to_hex(),
-                                              rev.digest_prefix()[0],
-                                              rev.digest_prefix()[1],
-                                              err);
+                                    weprintln!("Certificate {}: \
+                                                Failed to update \
+                                                revocation certificate \
+                                                {:02X}{:02X}: {}",
+                                               cert.keyid().to_hex(),
+                                               rev.digest_prefix()[0],
+                                               rev.digest_prefix()[1],
+                                               err);
                                 }
                             }
                         }
@@ -559,9 +516,9 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                 let sig = ua.binding_signature();
                 if sig.hash_algo() == HashAlgorithm::SHA1 {
                     diag!("Certificate {} contains a \
-                           User ID ({:?}) protected by SHA-1",
+                           User ID ({}) protected by SHA-1",
                           cert.keyid().to_hex(),
-                          String::from_utf8_lossy(ua.value()));
+                          ui::Safe(ua.userid()));
 
                     if !sha1_protected_userid {
                         sha1_protected_userid = true;
@@ -576,13 +533,12 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                             }
                             Err(err) => {
                                 unfixed_issue += 1;
-                                wprintln!("Certificate {}: User ID {}: \
-                                           Failed to update \
-                                           binding signature: {}",
-                                          cert.keyid().to_hex(),
-                                          String::from_utf8_lossy(
-                                              ua.value()),
-                                          err);
+                                weprintln!("Certificate {}: User ID {}: \
+                                            Failed to update \
+                                            binding signature: {}",
+                                           cert.keyid().to_hex(),
+                                           ui::Safe(ua.userid()),
+                                           err);
                             }
                         }
                     }
@@ -627,12 +583,12 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                                 Ok(sig) => updates.push(sig),
                                 Err(err) => {
                                     unfixed_issue += 1;
-                                    wprintln!("Certificate {}, key {}: \
-                                               Failed to update \
-                                               binding signature: {}",
-                                              cert.keyid().to_hex(),
-                                              ka.keyid().to_hex(),
-                                              err);
+                                    weprintln!("Certificate {}, key {}: \
+                                                Failed to update \
+                                                binding signature: {}",
+                                               cert.keyid().to_hex(),
+                                               ka.keyid().to_hex(),
+                                               err);
                                 }
                             }
                         }
@@ -676,8 +632,8 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                     backsigs.dedup();
 
                     if backsigs.len() > 1 {
-                        wprintln!("Warning: multiple cryptographically \
-                                   valid backsigs.");
+                        weprintln!("Warning: multiple cryptographically \
+                                    valid backsigs.");
                     }
 
                     if backsigs
@@ -715,12 +671,12 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                                 Ok(sig) => updates.push(sig),
                                 Err(err) => {
                                     unfixed_issue += 1;
-                                    wprintln!("Certificate {}, key: {}: \
-                                               Failed to update \
-                                               binding signature: {}",
-                                              cert.keyid().to_hex(),
-                                              ka.keyid().to_hex(),
-                                              err);
+                                    weprintln!("Certificate {}, key: {}: \
+                                                Failed to update \
+                                                binding signature: {}",
+                                               cert.keyid().to_hex(),
+                                               ka.keyid().to_hex(),
+                                               err);
                                 }
                             }
                         }
@@ -780,108 +736,102 @@ pub fn lint(sq: Sq, mut args: Command) -> Result<()> {
                     eprint!("GOOD");
                 }
             }
-            wprintln!(")");
+            weprintln!(")");
         }};
     }
 
-    if certs_with_issues > 0 {
-        wprintln!("Examined {} {}.",
-                  certs_valid + certs_invalid,
-                  pl(certs_valid + certs_invalid,
-                     "certificate", "certificates"));
+    if ! sq.quiet() {
+        weprintln!("Examined {} {}.",
+                   certs_valid + certs_invalid,
+                   pl(certs_valid + certs_invalid,
+                      "certificate", "certificates"));
 
-        if ! args.quiet {
-            err!(certs_invalid,
-                 "  {} {} invalid and {} not linted.",
-                 certs_invalid,
-                 pl(certs_invalid, "certificate is", "certificates are"),
-                 pl(certs_invalid, "was", "were"));
-            if certs_valid > 0 {
-                wprintln!("  {} {} linted.",
-                          certs_valid,
-                          pl(certs_valid,
-                             "certificate was", "certificates were"));
-                err!(certs_with_issues,
-                     "  {} of the {} certificates ({}%) \
-                      {} at least one issue.",
-                     certs_with_issues,
-                     certs_valid + certs_invalid,
-                     certs_with_issues * 100 / (certs_valid + certs_invalid),
-                     pl(certs_with_issues, "has", "have"));
-                wprintln!("{} of the linted certificates {} revoked.",
-                          certs_revoked,
-                          pl(certs_revoked, "was", "were"));
-                err!(certs_with_inadequota_revocations,
-                     "  {} of the {} certificates has revocation certificates \
-                      that are weaker than the certificate and should be \
-                      recreated.",
-                     certs_with_inadequota_revocations,
-                     certs_revoked);
-                wprintln!("{} of the linted certificates {} expired.",
-                          certs_expired,
-                          pl(certs_expired, "was", "were"));
-                wprintln!("{} of the non-revoked linted {} at least one non-revoked User ID:",
-                          certs_sp_sha1_userids,
-                          pl(certs_sp_sha1_userids,
-                             "certificate has", "certificates have"));
-                err!(certs_with_a_sha1_protected_userid,
-                     "  {} {} at least one User ID protected by SHA-1.",
-                     certs_with_a_sha1_protected_userid,
-                     pl(certs_with_a_sha1_protected_userid, "has", "have"));
-                err!(certs_with_only_sha1_protected_userids,
-                     "  {} {} all User IDs protected by SHA-1.",
-                     certs_with_only_sha1_protected_userids,
-                     pl(certs_with_only_sha1_protected_userids,
-                        "has", "have"));
-                wprintln!("{} of the non-revoked linted certificates {} at least one \
-                           non-revoked, live subkey:",
-                          certs_with_subkeys,
-                          pl(certs_with_subkeys,
-                             "has", "have"));
-                err!(certs_with_a_sha1_protected_binding_sig,
-                     "  {} {} at least one non-revoked, live subkey with \
-                      a binding signature that uses SHA-1.",
-                     certs_with_a_sha1_protected_binding_sig,
-                     pl(certs_with_a_sha1_protected_binding_sig,
-                        "has", "have"));
-                wprintln!("{} of the non-revoked linted certificates {} at least one non-revoked, live, \
-                           signing-capable subkey:",
-                          certs_with_signing_subkeys,
-                          pl(certs_with_signing_subkeys,
-                             "has", "have"));
-                err!(certs_with_sha1_protected_backsig,
-                     "  {} {} at least one non-revoked, live, signing-capable subkey \
-                      with a strong binding signature, but a backsig \
-                      that uses SHA-1.",
-                     certs_with_sha1_protected_backsig,
-                     pl(certs_with_sha1_protected_backsig,
-                        "certificate has", "certificates have"));
-            }
-        }
-
-        if args.fix {
-            if unfixed_issue == 0 {
-                if bad_input {
-                    exit(1);
-                } else {
-                    exit(0);
-                }
-            } else {
-                if ! args.quiet {
-                    err!(unfixed_issue,
-                         "Failed to fix {} {}.",
-                         unfixed_issue,
-                         pl(unfixed_issue, "issue", "issues"));
-                }
-                exit(3);
-            }
-        } else {
-            exit(2);
+        err!(certs_invalid,
+             "  {} {} invalid and {} not linted.",
+             certs_invalid,
+             pl(certs_invalid, "certificate is", "certificates are"),
+             pl(certs_invalid, "was", "were"));
+        if certs_valid > 0 {
+            weprintln!("  {} {} linted.",
+                       certs_valid,
+                       pl(certs_valid,
+                          "certificate was", "certificates were"));
+            err!(certs_with_issues,
+                 "  {} of the {} certificates ({}%) \
+                  {} at least one issue.",
+                 certs_with_issues,
+                 certs_valid + certs_invalid,
+                 certs_with_issues * 100 / (certs_valid + certs_invalid),
+                 pl(certs_with_issues, "has", "have"));
+            weprintln!("{} of the linted certificates {} revoked.",
+                       certs_revoked,
+                       pl(certs_revoked, "was", "were"));
+            err!(certs_with_inadequota_revocations,
+                 "  {} of the {} certificates has revocation certificates \
+                  that are weaker than the certificate and should be \
+                  recreated.",
+                 certs_with_inadequota_revocations,
+                 certs_revoked);
+            weprintln!("{} of the linted certificates {} expired.",
+                       certs_expired,
+                       pl(certs_expired, "was", "were"));
+            weprintln!("{} of the non-revoked linted {} at least one non-revoked User ID:",
+                       certs_sp_sha1_userids,
+                       pl(certs_sp_sha1_userids,
+                          "certificate has", "certificates have"));
+            err!(certs_with_a_sha1_protected_userid,
+                 "  {} {} at least one User ID protected by SHA-1.",
+                 certs_with_a_sha1_protected_userid,
+                 pl(certs_with_a_sha1_protected_userid, "has", "have"));
+            err!(certs_with_only_sha1_protected_userids,
+                 "  {} {} all User IDs protected by SHA-1.",
+                 certs_with_only_sha1_protected_userids,
+                 pl(certs_with_only_sha1_protected_userids,
+                    "has", "have"));
+            weprintln!("{} of the non-revoked linted certificates {} at least one \
+                        non-revoked, live subkey:",
+                       certs_with_subkeys,
+                       pl(certs_with_subkeys,
+                          "has", "have"));
+            err!(certs_with_a_sha1_protected_binding_sig,
+                 "  {} {} at least one non-revoked, live subkey with \
+                  a binding signature that uses SHA-1.",
+                 certs_with_a_sha1_protected_binding_sig,
+                 pl(certs_with_a_sha1_protected_binding_sig,
+                    "has", "have"));
+            weprintln!("{} of the non-revoked linted certificates {} at least one non-revoked, live, \
+                        signing-capable subkey:",
+                       certs_with_signing_subkeys,
+                       pl(certs_with_signing_subkeys,
+                          "has", "have"));
+            err!(certs_with_sha1_protected_backsig,
+                 "  {} {} at least one non-revoked, live, signing-capable subkey \
+                  with a strong binding signature, but a backsig \
+                  that uses SHA-1.",
+                 certs_with_sha1_protected_backsig,
+                 pl(certs_with_sha1_protected_backsig,
+                    "certificate has", "certificates have"));
         }
     }
 
-    if bad_input {
-        exit(1);
+    if args.fix {
+        if unfixed_issue > 0 {
+            if ! sq.quiet() {
+                err!(unfixed_issue,
+                     "Failed to fix {} {}.",
+                     unfixed_issue,
+                     pl(unfixed_issue, "issue", "issues"));
+            }
+            return Err(anyhow::anyhow!(
+                "Failed to fix {} {}",
+                unfixed_issue,
+                pl(unfixed_issue, "issue", "issues")));
+        }
+    } else if certs_with_issues > 0 {
+        return Err(anyhow::anyhow!("{} {} have at least one issue",
+                                   certs_with_issues,
+                                   pl(certs_with_issues,
+                                      "certificate", "certificates")));
     }
 
     Ok(())
