@@ -1,12 +1,8 @@
-use std::str::from_utf8;
 use std::sync::Arc;
 use std::time::SystemTime;
 
 use anyhow::Context;
 
-use anyhow::anyhow;
-use itertools::Itertools;
-
 use sequoia_openpgp as openpgp;
 use openpgp::cert::amalgamation::ValidAmalgamation;
 use openpgp::cert::UserIDRevocationBuilder;
@@ -14,7 +10,6 @@ use openpgp::packet::signature::subpacket::NotationData;
 use openpgp::packet::signature::subpacket::SubpacketTag;
 use openpgp::packet::signature::SignatureBuilder;
 use openpgp::packet::UserID;
-use openpgp::parse::Parse;
 use openpgp::policy::HashAlgoSecurity;
 use openpgp::policy::Policy;
 use openpgp::serialize::Serialize;
@@ -28,21 +23,24 @@ use sequoia_cert_store as cert_store;
 use cert_store::StoreUpdate;
 
 use crate::Sq;
+use crate::cli::key::userid::RevokeCommand;
+use crate::cli::types::userid_designator::ResolvedUserID;
 use crate::cli;
-use crate::cli::key::UseridRevokeCommand;
-use crate::cli::types::FileOrStdout;
-use crate::common::NULL_POLICY;
 use crate::common::RevocationOutput;
 use crate::common::get_secret_signer;
-use crate::common::userid::lint_userids;
-use crate::parse_notations;
+use crate::common::ui;
+use crate::common::userid::{
+    lint_emails,
+    lint_names,
+    lint_userids,
+};
+use crate::sq::TrustThreshold;
 
 /// Handle the revocation of a User ID
 struct UserIDRevocation {
     cert: Cert,
     revoker: Cert,
     revocation_packet: Packet,
-    userid: String,
     uid: UserID,
 }
 
@@ -50,8 +48,7 @@ impl UserIDRevocation {
     /// Create a new UserIDRevocation
     pub fn new(
         sq: &Sq,
-        userid: String,
-        force: bool,
+        uid: ResolvedUserID,
         cert: Cert,
         revoker: Option<Cert>,
         reason: ReasonForRevocation,
@@ -61,46 +58,8 @@ impl UserIDRevocation {
         let (revoker, mut signer)
             = get_secret_signer(sq, &cert, revoker.as_ref())?;
 
-        let uid = UserID::from(userid.as_str());
-
         let revocation_packet = {
             // Create a revocation for a User ID.
-
-            // Unless force is specified, we require the User ID to
-            // have a valid self signature under the Null policy.  We
-            // use the Null policy and not the standard policy,
-            // because it is still useful to revoke a User ID whose
-            // self signature is no longer valid.  For instance, the
-            // binding signature may use SHA-1.
-            if !force {
-                let valid_cert = cert.with_policy(NULL_POLICY, None)?;
-                let present = valid_cert
-                    .userids()
-                    .any(|u| u.userid() == &uid);
-
-                if !present {
-                    wprintln!(
-                        "User ID, cert: Cert, secret: Option<Cert>: '{}' not found.\nValid User IDs:",
-                        userid
-                    );
-                    let mut have_valid = false;
-                    for ua in valid_cert.userids() {
-                        if let Ok(u) = from_utf8(ua.userid().value()) {
-                            have_valid = true;
-                            wprintln!("  - {}", u);
-                        }
-                    }
-                    if !have_valid {
-                        wprintln!("  - Certificate has no valid User IDs.");
-                    }
-                    return Err(anyhow!(
-                        "The certificate does not contain the specified User \
-                        ID.  To create a revocation certificate for that User \
-                        ID anyways, specify '--force'"
-                    ));
-                }
-            }
-
             let mut rev = UserIDRevocationBuilder::new()
                 .set_reason_for_revocation(reason, message.as_bytes())?;
             rev = rev.set_signature_creation_time(sq.time)?;
@@ -115,7 +74,7 @@ impl UserIDRevocation {
             let rev = rev.build(
                 &mut signer,
                 &cert,
-                &uid,
+                uid.userid(),
                 None,
             )?;
             Packet::Signature(rev)
@@ -125,8 +84,7 @@ impl UserIDRevocation {
             cert,
             revoker,
             revocation_packet,
-            userid,
-            uid,
+            uid: uid.userid().clone(),
         })
     }
 }
@@ -144,14 +102,10 @@ impl RevocationOutput for UserIDRevocation
     }
 
     fn comment(&self) -> String {
-        let s = format!("Includes a revocation certificate for User ID {}",
-                        self.userid);
-        // Truncate it, if it is too long.
-        if s.len() > 70 {
-            format!("{:.70}", s)
-        } else {
-            s
-        }
+        format!("This is a revocation certificate for \
+                 the User ID {} of cert {}.",
+                ui::Safe(&self.uid),
+                self.cert.fingerprint())
     }
 
     fn revoker(&self) -> &Cert {
@@ -161,12 +115,11 @@ impl RevocationOutput for UserIDRevocation
 
 pub fn dispatch(
     sq: Sq,
-    command: cli::key::UseridCommand,
+    command: cli::key::userid::Command,
 ) -> Result<()> {
     match command {
-        cli::key::UseridCommand::Add(c) => userid_add(sq, c)?,
-        cli::key::UseridCommand::Revoke(c) => userid_revoke(sq, c)?,
-        cli::key::UseridCommand::Strip(c) => userid_strip(sq, c)?,
+        cli::key::userid::Command::Add(c) => userid_add(sq, c)?,
+        cli::key::userid::Command::Revoke(c) => userid_revoke(sq, c)?,
     }
 
     Ok(())
@@ -174,24 +127,22 @@ pub fn dispatch(
 
 fn userid_add(
     sq: Sq,
-    mut command: cli::key::UseridAddCommand,
+    mut command: cli::key::userid::AddCommand,
 ) -> Result<()> {
-    let cert = if let Some(file) = command.cert_file {
-        // If `--output` is not specified, default to writing to
-        // stdout, not to the certificate store.
-        if command.output.is_none() {
-            command.output = Some(FileOrStdout::new(None));
-        }
+    let cert = sq.resolve_cert(&command.cert, TrustThreshold::Full)?.0;
 
-        let input = file.open()?;
-        Cert::from_buffered_reader(input)?
-    } else if let Some(kh) = command.cert {
-        sq.lookup_one(&kh, None, false)?
-    } else {
-        panic!("--cert or --cert-file is required");
-    };
+    let mut signer = sq.get_primary_key(&cert, None)?;
 
-    let mut signer = sq.get_primary_key(&cert, None)?.0;
+    // Names, email addresses, and user IDs.
+    lint_names(&command.names)?;
+    for n in &command.names {
+        command.userid.push(UserID::from(n.as_str()));
+    }
+
+    lint_emails(&command.emails)?;
+    for n in &command.emails {
+        command.userid.push(UserID::from_address(None, None, n)?);
+    }
 
     // Make sure the user IDs are in canonical form.  If not, and
     // `--allow-non-canonical-userids` is not set, error out.
@@ -210,7 +161,9 @@ fn userid_add(
     if !exists.is_empty() {
         return Err(anyhow::anyhow!(
             "The certificate already contains the User ID(s) {}.",
-            exists.iter().map(|s| format!("{:?}", s)).join(", ")
+            exists.iter().map(|s| ui::Safe(*s).to_string())
+                .collect::<Vec<_>>()
+                .join(", "),
         ));
     }
 
@@ -320,8 +273,8 @@ fn userid_add(
     let cert = cert.insert_packets(add)?;
 
     if let Some(output) = command.output {
-        let mut sink = output.for_secrets().create_safe(sq.force)?;
-        if command.binary {
+        let mut sink = output.for_secrets().create_safe(&sq)?;
+        if false {
             cert.as_tsk().serialize(&mut sink)?;
         } else {
             cert.as_tsk().armored().serialize(&mut sink)?;
@@ -334,60 +287,6 @@ fn userid_add(
     Ok(())
 }
 
-fn userid_strip(
-    sq: Sq,
-    command: cli::key::UseridStripCommand,
-) -> Result<()> {
-    let input = command.input.open()?;
-    let key = Cert::from_buffered_reader(input)?;
-
-    let orig_cert_valid = key.with_policy(sq.policy, None).is_ok();
-
-    let strip: Vec<_> = command.userid;
-
-    // Make sure that each User ID that the user requested to remove exists in
-    // `key`, and *can* be removed.
-    let key_userids: Vec<_> =
-        key.userids().map(|u| u.userid().value()).collect();
-
-    let missing: Vec<_> = strip
-        .iter()
-        .filter(|s| !key_userids.contains(&s.value()))
-        .collect();
-    if !missing.is_empty() {
-        return Err(anyhow::anyhow!(
-            "The certificate doesn't contain the User ID(s) {}.",
-            missing.iter().map(|s| format!("{:?}", s)).join(", ")
-        ));
-    }
-
-    let cert = key.retain_userids(|uid| {
-        // Don't keep User IDs that were selected for removal
-        !strip.iter().any(|rm| rm == uid.component())
-    });
-
-    if orig_cert_valid {
-        if let Err(err) = cert.with_policy(sq.policy, None) {
-            wprintln!(
-                "Removing the User ID(s) has resulted in a invalid key:
-{}
-
-You could create a direct key signature or update the self
-signatures on other User IDs to make the key valid again.",
-                err
-            );
-        }
-    }
-
-    let mut sink = command.output.for_secrets().create_safe(sq.force)?;
-    if command.binary {
-        cert.as_tsk().serialize(&mut sink)?;
-    } else {
-        cert.as_tsk().armored().serialize(&mut sink)?;
-    }
-    Ok(())
-}
-
 /// Revoke a UserID of an existing primary key
 ///
 /// ## Errors
@@ -396,38 +295,36 @@ signatures on other User IDs to make the key valid again.",
 /// [`NotationData`] fails or if the eventual revocation fails.
 pub fn userid_revoke(
     sq: Sq,
-    mut command: UseridRevokeCommand,
+    command: RevokeCommand,
 ) -> Result<()> {
-    let cert = if let Some(file) = command.cert_file {
-        if command.output.is_none() {
-            // None means to write to the cert store.  When reading
-            // from a file, we want to write to stdout by default.
-            command.output = Some(FileOrStdout::new(None));
-        }
+    let cert = sq.resolve_cert(&command.cert, TrustThreshold::Full)?.0;
+    // To revoke a user ID, we require the certificate be valid under
+    // the current policy.  Users can still revoke user IDs whose
+    // binding signature relies on weak cryptography using
+    // `--add-user`.
+    let vcert = cert.with_policy(sq.policy, sq.time)
+        .with_context(|| {
+            format!("The certificate is not valid under the current \
+                     policy.  Consider revoking the whole certificate \
+                     using `sq key revoke`, or fixing it using \
+                     `sq cert lint` after verifying the certificate's \
+                     integrity.")
+        })?;
+    let userids = command.userids.resolve(&vcert)?;
+    assert_eq!(userids.len(), 1, "exactly one user ID enforced by clap");
+    let userid = userids.into_iter().next().unwrap();
 
-        let br = file.open()?;
-        Cert::from_buffered_reader(br)?
-    } else if let Some(kh) = command.cert {
-        sq.lookup_one(&kh, None, true)?
-    } else {
-        panic!("clap enforces --cert or --cert-file");
-    };
-
-    let revoker = if let Some(file) = command.revoker_file {
-        let br = file.open()?;
-        Some(Cert::from_buffered_reader(br)?)
-    } else if let Some(kh) = command.revoker {
-        Some(sq.lookup_one(&kh, None, true)?)
-    } else {
+    let revoker = if command.revoker.is_empty() {
         None
+    } else {
+        Some(sq.resolve_cert(&command.revoker, TrustThreshold::Full)?.0)
     };
 
-    let notations = parse_notations(command.notation)?;
+    let notations = command.signature_notations.parse()?;
 
     let revocation = UserIDRevocation::new(
         &sq,
-        command.userid,
-        sq.force,
+        userid,
         cert,
         revoker,
         command.reason.into(),
@@ -435,7 +332,7 @@ pub fn userid_revoke(
         &notations,
     )?;
 
-    revocation.write(&sq, command.output, command.binary)?;
+    revocation.write(&sq, command.output, false)?;
 
     Ok(())
 }
