@@ -3,7 +3,6 @@ use std::{
     fs::File,
     io,
     path::PathBuf,
-    process::exit,
 };
 use std::ops::Deref;
 use anyhow::Context;
@@ -17,30 +16,37 @@ use openpgp::{
         CertParser,
     },
     Fingerprint,
+    KeyHandle,
+    KeyID,
     packet::{
+        Packet,
         UserID,
         UserAttribute,
         Key,
+        Signature,
         Tag,
+        key,
+    },
+    parse::{
+        PacketParser,
+        PacketParserResult,
     },
     parse::Parse,
     serialize::Serialize,
 };
 
 use crate::{
-    Config,
+    Sq,
     Model,
     cli::types::FileOrStdout,
     output::KeyringListItem,
-    print_error_chain,
 };
 
 use crate::cli::keyring;
+use crate::cli::types::StdinWarning;
 
-mod lint;
-
-pub fn dispatch(config: Config, c: keyring::Command) -> Result<()> {
-    use crate::cli::keyring::Subcommands::*;
+pub fn dispatch(sq: Sq, c: keyring::Command) -> Result<()> {
+    use keyring::Subcommands::*;
     match c.subcommand {
         Filter(command) => {
             let any_uid_predicates =
@@ -82,11 +88,22 @@ pub fn dispatch(config: Config, c: keyring::Command) -> Result<()> {
             let any_ua_predicates = false;
             let ua_predicate = |_ua: &UserAttribute| false;
 
-            let any_key_predicates = ! command.handle.is_empty();
-            let key_predicate = |key: &Key<_, _>| {
+            let any_cert_predicates = ! command.cert.is_empty();
+            let cert_predicate = |key: &Key<_, key::PrimaryRole>| {
                 let mut keep = false;
 
-                for handle in &command.handle {
+                for handle in &command.cert {
+                    keep |= handle.aliases(key.key_handle());
+                }
+
+                keep
+            };
+
+            let any_key_predicates = ! command.key.is_empty();
+            let key_predicate = |key: &Key<_, key::SubordinateRole>| {
+                let mut keep = false;
+
+                for handle in &command.key {
                     keep |= handle.aliases(key.key_handle());
                 }
 
@@ -96,12 +113,15 @@ pub fn dispatch(config: Config, c: keyring::Command) -> Result<()> {
             let filter_fn = |c: Cert| -> Option<Cert> {
                 if ! (any_uid_predicates
                       || any_ua_predicates
+                      || any_cert_predicates
                       || any_key_predicates) {
                     // If there are no filters, pass it through.
                     Some(c)
                 } else if ! (c.userids().any(|c| uid_predicate(&c))
                              || c.user_attributes().any(|c| ua_predicate(&c))
-                             || c.keys().any(|c| key_predicate(c.key()))) {
+                             || cert_predicate(c.primary_key().key())
+                             || key_predicate(c.primary_key().key().role_as_subordinate())
+                             || c.keys().subkeys().any(|c| key_predicate(c.key()))) {
                     None
                 } else if command.prune_certs {
                     let c = c
@@ -112,12 +132,15 @@ pub fn dispatch(config: Config, c: keyring::Command) -> Result<()> {
                             ! any_ua_predicates || ua_predicate(&c)
                         })
                         .retain_subkeys(|c| {
-                            ! any_key_predicates
-                                || key_predicate(c.key().role_as_unspecified())
+                            ! any_key_predicates || key_predicate(c.key())
                         });
-                    if c.userids().count() == 0
-                        && c.user_attributes().count() == 0
-                        && c.keys().subkeys().count() == 0
+                    if (c.userids().next().is_none()
+                        && c.user_attributes().next().is_none()
+                        && c.keys().subkeys().next().is_none())
+                        || ((any_key_predicates
+                             && c.keys().subkeys().next().is_none())
+                            && (any_cert_predicates
+                                && ! cert_predicate(c.primary_key().key())))
                     {
                         // We stripped all components, omit this cert.
                         None
@@ -129,17 +152,17 @@ pub fn dispatch(config: Config, c: keyring::Command) -> Result<()> {
                 }
             };
 
-            filter(&config, command.input, command.output, filter_fn,
-                   command.binary, command.to_certificate)
+            filter(&sq, command.input, command.output, filter_fn,
+                   false, command.to_certificate)
         },
         Merge(c) =>
-            merge(&config, c.input, c.output, c.binary),
+            merge(&sq, c.input, c.output, false),
         List(c) => {
-            let mut input = c.input.open()?;
-            list(config, &mut input, c.all_userids)
+            let mut input = c.input.open("OpenPGP certificates")?;
+            list(sq, &mut input, c.all_userids)
         },
         Split(c) => {
-            let mut input = c.input.open()?;
+            let mut input = c.input.open("OpenPGP certificates")?;
             let prefix =
             // The prefix is either specified explicitly...
                 c.prefix.unwrap_or(
@@ -154,22 +177,13 @@ pub fn dispatch(config: Config, c: keyring::Command) -> Result<()> {
                         .unwrap_or_else(|| String::from("output"))
                     // ... finally, add a hyphen to the derived prefix.
                         + "-");
-            split(&mut input, &prefix, c.binary)
+            split(&mut input, &prefix, false)
         },
-        Lint(l) => {
-            match lint::lint(config, l) {
-                Ok(()) => Ok(()),
-                Err(e) => {
-                    print_error_chain(&e);
-                    exit(1);
-                },
-            }
-        }
     }
 }
 
 /// Joins certificates and keyrings into a keyring, applying a filter.
-fn filter<F>(config: &Config, inputs: Vec<PathBuf>, output: FileOrStdout,
+fn filter<F>(sq: &Sq, inputs: Vec<PathBuf>, output: FileOrStdout,
              mut filter: F,
              binary: bool,
              to_certificate: bool)
@@ -183,22 +197,18 @@ fn filter<F>(config: &Config, inputs: Vec<PathBuf>, output: FileOrStdout,
             for cert in CertParser::from_file(name.deref())? {
                 let cert = cert.context(
                     format!("Malformed certificate in keyring {:?}", name.display()))?;
-                if let Some(cert) = filter(cert) {
-                    certs.push(cert);
-                }
+                certs.push(cert);
             }
         }
     } else {
-        for cert in CertParser::from_reader(io::stdin())? {
+        for cert in CertParser::from_reader(StdinWarning::certs())? {
             let cert = cert.context("Malformed certificate in keyring")?;
-            if let Some(cert) = filter(cert) {
-                certs.push(cert);
-            }
+            certs.push(cert);
         }
     }
 
     let mut output = output.for_secrets().create_pgp_safe(
-        config.force,
+        &sq,
         binary,
         if ! to_certificate && certs.iter().any(|c| c.is_tsk()) {
             armor::Kind::SecretKey
@@ -222,19 +232,19 @@ fn filter<F>(config: &Config, inputs: Vec<PathBuf>, output: FileOrStdout,
 }
 
 /// Lists certs in a keyring.
-fn list(config: Config,
+fn list(sq: Sq,
         input: &mut (dyn io::Read + Sync + Send),
         list_all_uids: bool)
         -> Result<()>
 {
     let mut certs = vec![];
     let iter = CertParser::from_reader(input)?
-        .map(|item| KeyringListItem::from_cert_with_config(item, &config));
+        .map(|item| KeyringListItem::from_cert_with_sq(item, &sq));
     for item in iter {
         certs.push(item);
     }
-    let list = Model::keyring_list(config.output_version, certs, list_all_uids)?;
-    list.write(config.output_format, &mut std::io::stdout())?;
+    let list = Model::keyring_list(certs, list_all_uids)?;
+    list.write(&mut std::io::stdout())?;
     Ok(())
 }
 
@@ -324,33 +334,72 @@ fn split(input: &mut (dyn io::Read + Sync + Send), prefix: &str, binary: bool)
 }
 
 /// Merge multiple keyrings.
-fn merge(config: &Config, inputs: Vec<PathBuf>, output: FileOrStdout,
+fn merge(sq: &Sq, inputs: Vec<PathBuf>, output: FileOrStdout,
          binary: bool)
          -> Result<()>
 {
     let mut certs: BTreeMap<Fingerprint, Option<Cert>> = BTreeMap::new();
+    let mut revocations: Vec<(Signature, String)> = Vec::new();
 
-    if !inputs.is_empty() {
-        for name in inputs {
-            for cert in CertParser::from_file(&name)? {
-                let cert = cert.context(
-                    format!("Malformed certificate in keyring {:?}", name))?;
-                match certs.entry(cert.fingerprint()) {
-                    e @ Entry::Vacant(_) => {
-                        e.or_insert(Some(cert));
-                    }
-                    Entry::Occupied(mut e) => {
-                        let e = e.get_mut();
-                        let curr = e.take().unwrap();
-                        *e = Some(curr.merge_public_and_secret(cert)
-                            .expect("Same certificate"));
-                    }
-                }
+    let inputs: Box<dyn Iterator<Item = _>> = if inputs.is_empty() {
+        Box::new(std::iter::once(
+            ("stdin".to_string(),
+             PacketParser::from_reader(StdinWarning::certs()))))
+    } else {
+        Box::new(
+            inputs.into_iter()
+                .map(|name| {
+                    (name.display().to_string(),
+                     PacketParser::from_file(&name))
+                }))
+    };
+
+    for (name, result) in inputs {
+        let parser = result.with_context(|| format!("Opening {}", name))?;
+
+        // First see if we have a bare revocation certificate.
+        let mut is_sig = false;
+        if let PacketParserResult::Some(ref pp) = parser {
+            if let Packet::Signature(_) = pp.packet {
+                is_sig = true;
             }
         }
-    } else {
-        for cert in CertParser::from_reader(io::stdin())? {
-            let cert = cert.context("Malformed certificate in keyring")?;
+        if is_sig {
+            // The first packet is a sig.  Make sure it is *only* a
+            // sig.
+            let sig = if let PacketParserResult::Some(pp) = parser {
+                let (packet, next_ppr) = pp.next()?;
+
+                let sig = if let Packet::Signature(sig) = packet {
+                    sig
+                } else {
+                    return Err(anyhow::anyhow!(
+                        "{}: Not a revocation certificate: got a {}.",
+                        name, packet.tag()));
+                };
+
+                if let PacketParserResult::Some(_) = next_ppr {
+                    return Err(anyhow::anyhow!(
+                        "{}: Not a revocation certificate: \
+                         got more than one packet.",
+                        name));
+                }
+
+                sig
+            } else {
+                return Err(anyhow::anyhow!(
+                    "{}: Not a bare revocation certificate.",
+                    name));
+            };
+
+            revocations.push((sig.clone(), name));
+            continue;
+        }
+
+        // Parse it like its a keyring.
+        for cert in CertParser::from(parser) {
+            let cert = cert.context(
+                format!("Read a malformed certificate from {:?}", name))?;
             match certs.entry(cert.fingerprint()) {
                 e @ Entry::Vacant(_) => {
                     e.or_insert(Some(cert));
@@ -365,8 +414,93 @@ fn merge(config: &Config, inputs: Vec<PathBuf>, output: FileOrStdout,
         }
     }
 
+    if ! revocations.is_empty() {
+        // A map from key ID to fingerprint.
+        //
+        // This doesn't deal with colliding key IDs, but second
+        // pre-images are not feasible, so this isn't a problem in
+        // practice.
+        let by_keyid: BTreeMap<KeyID, Fingerprint>
+            = BTreeMap::from_iter(certs.values().filter_map(|cert| {
+                cert.as_ref().map(|cert| {
+                    (cert.keyid(), cert.fingerprint())
+                })
+            }));
+
+        let mut die = false;
+        let mut missing = Vec::new();
+
+        'next_rev: for (sig, name) in revocations {
+            let issuers = sig.get_issuers();
+
+            let mut bad = None;
+
+            for issuer in issuers.iter() {
+                let cert = match issuer {
+                    KeyHandle::Fingerprint(fpr) => {
+                        certs.get_mut(fpr)
+                    }
+                    KeyHandle::KeyID(keyid) => {
+                        by_keyid.get(&keyid)
+                            .and_then(|fpr| certs.get_mut(fpr))
+                    }
+                };
+
+                let cert = if let Some(Some(cert)) = cert {
+                    cert
+                } else {
+                    continue;
+                };
+
+                match sig.clone().verify_primary_key_revocation(
+                    cert.primary_key().key(),
+                    cert.primary_key().key())
+                {
+                    Ok(()) => {
+                        *cert = cert.clone().insert_packets(sig.clone())?;
+                        continue 'next_rev;
+                    }
+                    Err(err) => {
+                        bad = Some((issuer, name.clone(), err));
+                    }
+                }
+            }
+
+            if let Some((_sig, name, err)) = bad {
+                weprintln!("Could not add revocation certificate from {} \
+                            to certificate: {}",
+                           name, err);
+                die = true;
+            } else {
+                missing.push((issuers[0].clone(), name.clone()));
+                die = true;
+            }
+        }
+
+        match missing.as_slice() {
+            [] => (),
+            [(issuer, name)] => {
+                weprintln!("Couldn't merge revocation certificate \
+                            from {}: missing {}.",
+                           name, issuer);
+            }
+            _ => {
+                weprintln!("Couldn't some merge revocation certificates:");
+                for (issuer, name) in missing.into_iter() {
+                    weprintln!("  - {}: missing {}",
+                               name, issuer);
+                }
+            }
+        }
+
+        if die {
+            return Err(anyhow::anyhow!(
+                "Failed to merge some revocation certificates"));
+        }
+    }
+
     let mut output = output.for_secrets().create_pgp_safe(
-        config.force,
+        &sq,
         binary,
         if certs.values().any(|c| c.as_ref().map(Cert::is_tsk).unwrap_or(false))
         {
