@@ -44,8 +44,11 @@ pub trait RevocationOutput {
     fn write(&self, sq: &Sq, output: Option<FileOrStdout>, binary: bool)
         -> Result<()>
     {
+        // Target width of comments.
+        const COMMENT_WIDTH: usize = 70;
+
         if let Some(output) = output {
-            let mut output = output.create_safe(sq.force)?;
+            let mut output = output.create_safe(sq)?;
 
             // First, build a minimal revocation certificate containing
             // the primary key, the revoked component, and the revocation
@@ -60,28 +63,23 @@ pub trait RevocationOutput {
                 let mut more: Vec<String> = vec![];
 
                 // First, the thing that is being revoked.
-                more.push(self.comment());
+                textwrap::wrap(&self.comment(), COMMENT_WIDTH)
+                    .into_iter().for_each(|line| more.push(line.into()));
 
                 let revoker = self.revoker();
 
                 let first_party_issuer = revoker.fingerprint() == cert.fingerprint();
                 if ! first_party_issuer {
                     // Then if it was issued by a third-party.
-                    more.push("issued by".to_string());
-                    more.push(revoker.fingerprint().to_spaced_hex());
-                    // This information may be published so only consider
-                    // self-signed user IDs to avoid leaking information
-                    // about the user's web of trust.
-                    let sanitized_uid = sq.best_userid(revoker, false);
-                    // Truncate it, if it is too long.
-                    more.push(format!("{:.70}", sanitized_uid));
+                    more.push("".into());
+                    more.push(format!("This revocation is issued \
+                                       by the cert {} ({}).",
+                                      revoker.fingerprint(),
+                                      sq.best_userid(revoker, false)));
                 }
 
-                let headers = &cert.armor_headers();
-                let headers: Vec<(&str, &str)> = headers
-                    .iter()
-                    .map(|s| ("Comment", s.as_str()))
-                    .chain(more.iter().map(|value| ("Comment", value.as_str())))
+                let headers: Vec<(&str, &str)> = more.iter()
+                    .map(|value| ("Comment", value.as_str()))
                     .collect();
 
                 let mut writer =
@@ -127,7 +125,7 @@ pub fn get_secret_signer<'a>(
 
     if let Some(secret) = secret {
         match sq.get_primary_key(secret, flags) {
-            Ok((key, _password)) => Ok((secret.clone(), key)),
+            Ok(key) => Ok((secret.clone(), key)),
             Err(err) => {
                 if ! sq.time_is_now {
                     return Err(err.context(format!("\
@@ -145,7 +143,7 @@ does not contain a certification key with secret key material")));
         }
     } else {
         match sq.get_primary_key(cert, flags) {
-            Ok((key, _password)) => Ok((cert.clone(), key)),
+            Ok(key) => Ok((cert.clone(), key)),
             Err(err) => {
                 if ! sq.time_is_now {
                     return Err(err.context(format!("\
