@@ -20,8 +20,6 @@ use sequoia_openpgp as openpgp;
 use openpgp::Result;
 use openpgp::Cert;
 use openpgp::parse::Parse;
-use openpgp::packet::signature::subpacket::NotationData;
-use openpgp::packet::signature::subpacket::NotationDataFlags;
 use openpgp::cert::prelude::*;
 
 use clap::FromArgMatches;
@@ -48,6 +46,7 @@ use cli::types::Version;
 use cli::types::paths::StateDirectory;
 
 mod commands;
+pub mod compat;
 pub mod config;
 pub mod output;
 pub use output::Model;
@@ -130,8 +129,8 @@ where
 #[allow(dead_code)]
 fn help_warning(arg: &str) {
     if arg == "help" {
-        wprintln!("Warning: \"help\" is not a subcommand here.  \
-                   Did you mean --help?");
+        weprintln!("Warning: \"help\" is not a subcommand here.  \
+                    Did you mean --help?");
     }
 }
 
@@ -167,7 +166,7 @@ fn real_main() -> Result<()> {
                     usize::from_str(env!("CARGO_PKG_VERSION_MINOR")).unwrap(),
                     usize::from_str(env!("CARGO_PKG_VERSION_PATCH")).unwrap());
 
-                if ! version.is_acceptable_for(&cli_version) {
+                if ! cli_version.is_acceptable_for(&version) {
                     return Err(anyhow::anyhow!(
                         "The required CLI version, {}, is not compatible with \
                          this version of sq, which implements version {} of \
@@ -304,6 +303,9 @@ fn real_main() -> Result<()> {
         Default::default()
     };
 
+    config.init_verbose(c.verbose, matches.value_source("verbose"));
+    config.init_quiet(c.quiet, matches.value_source("quiet"));
+
     let time_is_now = c.time.is_none();
     let time: SystemTime = if let Some(t) = c.time.as_ref() {
         t.to_system_time(std::time::SystemTime::now())?
@@ -339,8 +341,6 @@ fn real_main() -> Result<()> {
     let sq = Sq {
         config_file,
         config,
-        verbose: c.verbose,
-        quiet: c.quiet,
         overwrite: c.overwrite,
         batch: c.batch,
         time,
@@ -395,7 +395,7 @@ fn real_main() -> Result<()> {
                     err.print()?;
 
                     if let Some(examples) = command.get_after_help() {
-                        wprintln!("\n{}", examples);
+                        weprintln!("\n{}", examples);
                     }
                 } else {
                     // Ummm... something went wrong: we should be able
@@ -420,37 +420,6 @@ fn real_main() -> Result<()> {
     }
 }
 
-fn parse_notations<N>(n: N) -> Result<Vec<(bool, NotationData)>>
-where
-    N: AsRef<[String]>,
-{
-    let n = n.as_ref();
-    assert_eq!(n.len() % 2, 0, "notations must be pairs of key and value");
-
-    // Each --notation takes two values.  Iterate over them in chunks of 2.
-    let notations: Vec<(bool, NotationData)> = n
-        .chunks(2)
-        .map(|arg_pair| {
-            let name = &arg_pair[0];
-            let value = &arg_pair[1];
-
-            let (critical, name) = match name.strip_prefix('!') {
-                Some(name) => (true, name),
-                None => (false, name.as_str()),
-            };
-
-            let notation_data = NotationData::new(
-                name,
-                value,
-                NotationDataFlags::empty().set_human_readable(),
-            );
-            (critical, notation_data)
-        })
-        .collect();
-
-    Ok(notations)
-}
-
 // Sometimes the same error cascades, e.g.:
 //
 // ```
@@ -475,10 +444,10 @@ fn error_chain(err: &anyhow::Error) -> Vec<String> {
 
 /// Prints the error and causes, if any.
 pub fn print_error_chain(err: &anyhow::Error) {
-    wprintln!();
-    wprintln!(initial_indent="  Error: ", "{}", err);
+    weprintln!();
+    weprintln!(initial_indent="  Error: ", "{}", err);
     err.chain().skip(1).for_each(
-        |cause| wprintln!(initial_indent="because: ", "{}", cause));
+        |cause| weprintln!(initial_indent="because: ", "{}", cause));
 }
 
 /// Returns the error chain as a string.
@@ -559,7 +528,7 @@ fn print_examples(cli: &clap::Command, usage: impl ToString) -> Result<()> {
 
     // And print the examples, if any.
     if let Some(examples) = cmd.get_after_help() {
-        wprintln!("\n{}", examples);
+        weprintln!("\n{}", examples);
     }
 
     Ok(())
