@@ -10,9 +10,11 @@ use openpgp::types::SignatureType;
 use openpgp::Cert;
 use openpgp::Result;
 
-use super::common::compare_notations;
-use super::common::Sq;
+use super::common::NO_USERIDS;
 use super::common::STANDARD_POLICY;
+use super::common::Sq;
+use super::common::UserIDArg;
+use super::common::compare_notations;
 use super::common::time_as_string;
 
 #[test]
@@ -51,7 +53,7 @@ fn sq_key_subkey() -> Result<()> {
                 ]);
             } else {
                 cmd.args([
-                    "--force",
+                    "--overwrite",
                     "--output",
                     &modified_cert_path.to_string_lossy(),
                     "--cert-file", &cert_path.to_string_lossy(),
@@ -89,7 +91,7 @@ fn sq_key_subkey_add_with_password() -> Result<()> {
         "--cannot-authenticate",
         "--cannot-encrypt",
         "--new-password-file", &path2.display().to_string(),
-    ], &[]);
+    ], NO_USERIDS);
 
     assert!(cert.is_tsk());
     assert_eq!(cert.keys().subkeys().count(), 0);
@@ -251,9 +253,9 @@ fn sq_key_subkey_revoke() -> Result<()> {
                 "key",
                 "subkey",
                 "revoke",
-                &subkey_fingerprint.to_string(),
-                reason_str,
-                message,
+                "--key", &subkey_fingerprint.to_string(),
+                "--reason", reason_str,
+                "--message", message,
             ]);
 
             if keystore {
@@ -266,7 +268,7 @@ fn sq_key_subkey_revoke() -> Result<()> {
             }
 
             for (k, v) in notations {
-                cmd.args(["--notation", k, v]);
+                cmd.args(["--signature-notation", k, v]);
             }
             if let Some(time) = revocation_time {
                 cmd.args([
@@ -276,7 +278,8 @@ fn sq_key_subkey_revoke() -> Result<()> {
             }
             let output = cmd.output()?;
             if !output.status.success() {
-                panic!("sq exited with non-zero status code: {:?}", output.stderr);
+                panic!("sq exited with non-zero status code: {}",
+                       String::from_utf8_lossy(&output.stderr));
             }
 
             // whether we found a revocation signature
@@ -347,6 +350,42 @@ fn sq_key_subkey_revoke() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn sq_key_subkey_revoke_multiple() -> Result<()> {
+    let sq = Sq::new();
+
+    let (cert, cert_path, _rev_path)
+        = sq.key_generate(&[], &[UserIDArg::Email("alice@example.org")]);
+    assert!(cert.keys().subkeys().count() > 0);
+    sq.key_import(cert_path);
+
+    let mut cmd = sq.command();
+    cmd.args([
+        "key", "subkey", "revoke",
+        "--cert", &cert.fingerprint().to_string(),
+    ]);
+
+    for subkey in cert.keys().subkeys() {
+        cmd.args(["--key", &subkey.key().fingerprint().to_string(),]);
+    }
+
+    cmd.args(["--reason", "retired", "--message", "rotation"]);
+    sq.run(cmd, Some(true));
+
+    let revoked = sq.cert_export(cert.key_handle());
+    assert_eq!(cert.keys().subkeys().count(), revoked.keys().subkeys().count());
+
+    let vrevoked = revoked.with_policy(STANDARD_POLICY, sq.now())?;
+    for subkey in vrevoked.keys().subkeys() {
+        use sequoia_openpgp::cert::amalgamation::ValidAmalgamation;
+        assert!(matches!(subkey.revocation_status(),
+                         RevocationStatus::Revoked(_)));
+    }
+
+    Ok(())
+}
+
+
 #[test]
 fn sq_key_subkey_revoke_thirdparty() -> Result<()> {
     let sq = Sq::new();
@@ -467,9 +506,9 @@ fn sq_key_subkey_revoke_thirdparty() -> Result<()> {
                 "key",
                 "subkey",
                 "revoke",
-                &subkey_fingerprint.to_string(),
-                reason_str,
-                message,
+                "--key", &subkey_fingerprint.to_string(),
+                "--reason", reason_str,
+                "--message", message,
             ]);
 
             if keystore {
@@ -484,7 +523,7 @@ fn sq_key_subkey_revoke_thirdparty() -> Result<()> {
             }
 
             for (k, v) in notations {
-                cmd.args(["--notation", k, v]);
+                cmd.args(["--signature-notation", k, v]);
             }
             if let Some(time) = revocation_time {
                 cmd.args([
