@@ -121,32 +121,45 @@ where
             cmd.arg(&target.to_string());
         }
         if let Some(userid) = userid {
-            userid.as_arg(&mut cmd);
+            if command == "list" {
+                userid.as_cert_designator(&mut cmd);
+            } else {
+                userid.as_arg(&mut cmd);
+            }
         }
         for arg in args {
             cmd.arg(arg);
         }
         cmd.args(&["--amount", &format!("{}", amount)]);
 
+        eprintln!("Running: {}",
+                  std::iter::once(cmd.get_program())
+                  .chain(cmd.get_args())
+                  .map(|arg| {
+                      let arg = arg.to_string_lossy();
+                      if arg.contains(" ") {
+                          format!("{:?}", arg)
+                      } else {
+                          arg.into_owned()
+                      }
+                  })
+                  .collect::<Vec<_>>()
+                  .join(" "));
+
         if success {
             let assertion = cmd.assert();
             let assertion = assertion.success();
 
             if let Some(output) = output.get(outputformat) {
                 for (expected_occurrences, s) in output {
-                    let haystack =
-                        if outputformat == &OutputFormat::HumanReadable {
-                            &assertion.get_output().stderr
-                        } else {
-                            &assertion.get_output().stdout
-                        };
+                    let haystack = &assertion.get_output().stdout;
                     let occurrences =
                         s.find_iter(haystack.as_ref()).count();
 
                     assert_eq!(
                         occurrences, *expected_occurrences,
-                        "Failed to find: '{}' {} times\n\
-                         in output:\n\
+                        "Failed to find: '{}' {} times \
+                         in output:\n\n\
                          {}",
                         s, expected_occurrences,
                         String::from_utf8_lossy(haystack),
@@ -159,19 +172,14 @@ where
 
             if let Some(output) = output.get(outputformat) {
                 for (expected_occurrences, s) in output {
-                    let haystack =
-                        if outputformat == &OutputFormat::HumanReadable {
-                            &assertion.get_output().stderr
-                        } else {
-                            &assertion.get_output().stdout
-                        };
+                    let haystack = &assertion.get_output().stdout;
                     let occurrences =
                         s.find_iter(haystack.as_ref()).count();
 
                     assert_eq!(
                         occurrences, *expected_occurrences,
-                        "Failed to find: '{}' {} times\n\
-                         in output:\n\
+                        "Failed to find: '{}' {} times \
+                         in output:\n\n\
                          {}",
                         s, expected_occurrences,
                         String::from_utf8_lossy(haystack),
@@ -703,7 +711,7 @@ fn lookup() -> Result<()> {
 
     let human_output = [
         (1, format!("- {} {}", HR_OK, &dave_uid)),
-        (1, format!("{}{} (\"{}\")", HR_PATH, &alice_fpr, &alice_uid)),
+        (1, format!("{}{} ({})", HR_PATH, &alice_fpr, &alice_uid)),
     ];
     test(
         keyring,
