@@ -19,14 +19,10 @@ use openpgp::policy::StandardPolicy;
 use openpgp::serialize::stream::{Message, Signer, Compressor, LiteralWriter};
 use openpgp::serialize::Serialize;
 
-use super::common::Sq;
+use super::common::*;
 
 const P: &StandardPolicy = &StandardPolicy::new();
 
-fn artifact(filename: &str) -> String {
-    format!("tests/data/{}", filename)
-}
-
 #[test]
 fn sq_sign() {
     let sq = Sq::new();
@@ -34,13 +30,8 @@ fn sq_sign() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign message.
-    sq.command()
-        .arg("sign")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton-private.pgp")])
-        .args(["--output", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.sign(artifact("keys/dennis-simon-anton-private.pgp"), None,
+            &artifact("messages/a-cypherpunks-manifesto.txt"), sig.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -67,12 +58,8 @@ fn sq_sign() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify signed message.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .arg(&*sig.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::Message, &sig, None);
 }
 
 #[test]
@@ -82,16 +69,11 @@ fn sq_sign_with_notations() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign message.
-    sq.command()
-        .arg("sign")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton-private.pgp")])
-        .args(["--output", &sig.to_string_lossy()])
-        .args(["--notation", "foo", "bar"])
-        .args(["--notation", "!foo", "xyzzy"])
-        .args(["--notation", "hello@example.org", "1234567890"])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.sign_args(&["--signature-notation", "foo", "bar",
+                   "--signature-notation", "!foo", "xyzzy",
+                   "--signature-notation", "hello@example.org", "1234567890"],
+                 artifact("keys/dennis-simon-anton-private.pgp"), None,
+                 &artifact("messages/a-cypherpunks-manifesto.txt"), sig.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -143,13 +125,9 @@ fn sq_sign_with_notations() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify signed message.
-    sq.command()
-        .args(["--known-notation", "foo"])
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .arg(&*sig.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp"),
+                "--known-notation", "foo"],
+              Verify::Message, &sig, None);
 }
 
 #[test]
@@ -159,13 +137,8 @@ fn sq_sign_append() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Sign message.
-    sq.command()
-        .arg("sign")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton-private.pgp")])
-        .args(["--output", &sig0.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.sign(artifact("keys/dennis-simon-anton-private.pgp"), None,
+            &artifact("messages/a-cypherpunks-manifesto.txt"), sig0.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -192,24 +165,15 @@ fn sq_sign_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify signed message.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::Message, &sig0, None);
 
     // Now add a second signature with --append.
     let sig1 = tmp_dir.path().join("sig1");
-    sq.command()
-        .arg("sign")
-        .arg("--append")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
-        .arg("--output")
-        .arg(&*sig1.to_string_lossy())
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.sign_args(
+        &["--append"],
+        artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+        None, sig0.as_path(), sig1.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -249,18 +213,10 @@ fn sq_sign_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both signatures of the signed message.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .arg(&*sig1.to_string_lossy())
-        .assert()
-        .success();
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
-        .arg(&*sig1.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::Message, &sig1, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")],
+              Verify::Message, &sig1, None);
 }
 
 #[test]
@@ -318,19 +274,16 @@ fn sq_sign_append_on_compress_then_sign() {
     }
 
     // Verify signed message.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::Message, &sig0, None);
 
     // Now add a second signature with --append.
     let sig1 = tmp_dir.path().join("sig1");
     sq.command()
         .arg("sign")
+        .arg("--message")
         .arg("--append")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .arg("--signer-file").arg(artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"))
         .arg("--output")
         .arg(&*sig1.to_string_lossy())
         .arg(&*sig0.to_string_lossy())
@@ -378,19 +331,10 @@ fn sq_sign_append_on_compress_then_sign() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both signatures of the signed message.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
-
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::Message, &sig0, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")],
+              Verify::Message, &sig0, None);
 }
 
 #[test]
@@ -400,14 +344,10 @@ fn sq_sign_detached() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign detached.
-    sq.command()
-        .arg("sign")
-        .arg("--detached")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton-private.pgp")])
-        .args(["--output", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.sign_detached(&[],
+                     artifact("keys/dennis-simon-anton-private.pgp"),
+                     &artifact("messages/a-cypherpunks-manifesto.txt"),
+                     sig.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -423,13 +363,9 @@ fn sq_sign_detached() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP SIGNATURE-----\n\n"));
 
     // Verify detached.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .args(["--detached", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::SignatureFile(sig),
+              &artifact("messages/a-cypherpunks-manifesto.txt"), None);
 }
 
 #[test]
@@ -439,14 +375,10 @@ fn sq_sign_detached_append() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign detached.
-    sq.command()
-        .arg("sign")
-        .arg("--detached")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton-private.pgp")])
-        .args(["--output", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.sign_detached(&[],
+                     artifact("keys/dennis-simon-anton-private.pgp"),
+                     &artifact("messages/a-cypherpunks-manifesto.txt"),
+                     sig.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -462,34 +394,24 @@ fn sq_sign_detached_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP SIGNATURE-----\n\n"));
 
     // Verify detached.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .args(["--detached", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::SignatureFile(sig.clone()),
+              &artifact("messages/a-cypherpunks-manifesto.txt"), None);
 
     // Check that we don't blindly overwrite signatures.
-    sq.command()
-        .arg("sign")
-        .arg("--detached")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
-        .args(["--output", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .failure();
+    sq.try_sign_detached(&[],
+                         artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+                         &artifact("messages/a-cypherpunks-manifesto.txt"),
+                         sig.as_path())
+        .unwrap_err();
 
     // Now add a second signature with --append.
-    sq.command()
-        .arg("sign")
-        .arg("--detached")
-        .arg("--append")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
-        .args(["--output", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    eprintln!("now");
+    sq.sign_detached(&["--append"],
+                     artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+                     &artifact("messages/a-cypherpunks-manifesto.txt"),
+                     sig.as_path());
+    eprintln!("done");
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -510,27 +432,18 @@ fn sq_sign_detached_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP SIGNATURE-----\n\n"));
 
     // Verify both detached signatures.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/dennis-simon-anton.pgp")])
-        .args(["--detached", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
-
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
-        .args(["--detached", &sig.to_string_lossy()])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/dennis-simon-anton.pgp")],
+              Verify::SignatureFile(sig.clone()),
+              &artifact("messages/a-cypherpunks-manifesto.txt"), None);
+    sq.verify(&["--signer-file", &artifact_s("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")],
+              Verify::SignatureFile(sig.clone()),
+              &artifact("messages/a-cypherpunks-manifesto.txt"), None);
 
     // Finally, check that we don't truncate the file if something
     // goes wrong.
     sq.command()
         .arg("sign")
-        .arg("--detached")
+        .arg("--signature-file")
         .arg("--append")
         .arg("--signer-file") // Not a private key => signing will fail.
         .arg(&artifact("keys/erika-corinna-daniela-simone-antonia-nistp521.pgp"))
@@ -565,14 +478,11 @@ fn sq_sign_append_a_notarization() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Now add a third signature with --append to a notarized message.
-    sq.command()
-        .arg("sign")
-        .arg("--append")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
-        .args(["--output", &sig0.to_string_lossy()])
-        .arg(&artifact("messages/signed-1-notarized-by-ed25519.pgp"))
-        .assert()
-        .success();
+    sq.sign_detached(
+        &["--append"],
+        artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+        &artifact("messages/signed-1-notarized-by-ed25519.pgp"),
+        sig0.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -624,24 +534,12 @@ fn sq_sign_append_a_notarization() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both notarizations and the signature.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/neal.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/neal.pgp")],
+              Verify::Message, &sig0, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp")],
+              Verify::Message, &sig0, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")],
+              Verify::Message, &sig0, None);
 }
 
 #[ignore]
@@ -652,14 +550,12 @@ fn sq_sign_notarize() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Now add a third signature with --append to a notarized message.
-    sq.command()
-        .arg("sign")
-        .arg("--notarize")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
-        .args(["--output", &sig0.to_string_lossy()])
-        .arg(&artifact("messages/signed-1.gpg"))
-        .assert()
-        .success();
+    sq.sign_args(
+        &["--notarize"],
+        artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+        None,
+        &artifact("messages/signed-1.gpg"),
+        sig0.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -699,18 +595,10 @@ fn sq_sign_notarize() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both notarizations and the signature.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/neal.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/neal.pgp")],
+              Verify::Message, &sig0, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")],
+              Verify::Message, &sig0, None);
 }
 
 #[ignore]
@@ -721,14 +609,12 @@ fn sq_sign_notarize_a_notarization() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Now add a third signature with --append to a notarized message.
-    sq.command()
-        .arg("sign")
-        .arg("--notarize")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
-        .args(["--output", &sig0.to_string_lossy()])
-        .arg(&artifact("messages/signed-1-notarized-by-ed25519.pgp"))
-        .assert()
-        .success();
+    sq.sign_args(
+        &["--notarize"],
+        artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+        None,
+        &artifact("messages/signed-1-notarized-by-ed25519.pgp"),
+        sig0.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -780,24 +666,12 @@ fn sq_sign_notarize_a_notarization() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both notarizations and the signature.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/neal.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
-        .arg(&*sig0.to_string_lossy())
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &artifact_s("keys/neal.pgp")],
+              Verify::Message, &sig0, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp")],
+              Verify::Message, &sig0, None);
+    sq.verify(&["--signer-file", &artifact_s("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")],
+              Verify::Message, &sig0, None);
 }
 
 #[test]
@@ -828,7 +702,7 @@ fn sq_multiple_signers() -> Result<()> {
             "sign",
             "--signer-file", alice_pgp.to_str().unwrap(),
             "--signer-file", &bob_pgp.to_str().unwrap(),
-            "--detached",
+            "--signature-file=-",
         ])
         .write_stdin(&b"foo"[..])
         .assert().try_success()?;
@@ -879,35 +753,23 @@ fn sq_sign_using_cert_store() -> Result<()> {
     let certd = dir.path().join("cert.d").display().to_string();
     std::fs::create_dir(&certd).expect("mkdir works");
 
-    let alice_pgp = dir.path().join("alice.pgp").display().to_string();
-    let msg_pgp = dir.path().join("msg.pgp").display().to_string();
+    let msg_pgp = dir.path().join("msg.pgp");
 
     // Generate a key.
-    let mut cmd = sq.command();
-    cmd.args(["--cert-store", &certd,
-              "key", "generate", "--without-password",
-              "--expiration", "never",
-              "--userid", "<alice@example.org>",
-              "--output", &alice_pgp]);
-    cmd.assert().success();
-
-    let alice = Cert::from_file(&alice_pgp)?;
+    let (alice, alice_pgp, _rev) =
+        sq.key_generate(&["--expiration", "never"], &["<alice@example.org>"]);
 
     // Import it.
     let mut cmd = sq.command();
     cmd.args(["--cert-store", &certd,
-              "cert", "import", &alice_pgp]);
-    cmd.assert().success();
-
+              "cert", "import"]);
+    cmd.arg(&alice_pgp);
+    sq.run(cmd, true);
 
     // Sign a message.
-    sq.command()
-        .arg("sign")
-        .args(["--signer-file", &alice_pgp])
-        .args(["--output", &msg_pgp])
-        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
-        .assert()
-        .success();
+    sq.sign(&alice_pgp, None,
+            &artifact("messages/a-cypherpunks-manifesto.txt"),
+            msg_pgp.as_path());
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -940,52 +802,35 @@ fn sq_sign_using_cert_store() -> Result<()> {
 
     // Verify the signed message.  First, we specify the certificate
     // explicitly.
-    sq.command()
-        .arg("verify")
-        .args(["--signer-file", &alice_pgp])
-        .arg(&msg_pgp)
-        .assert()
-        .success();
+    sq.verify(&["--signer-file", &alice_pgp.display().to_string()],
+              Verify::Message, &msg_pgp, None);
 
     // Verify the signed message.  Now, we don't specify the
     // certificate or use a certificate store.
-    let mut cmd = sq.command();
-    cmd.arg("verify")
-        .arg(&msg_pgp);
-    let output = cmd.output().expect("success");
-    let stdout = String::from_utf8_lossy(&output.stdout);
-    let stderr = String::from_utf8_lossy(&output.stderr);
-    assert!(! output.status.success(),
-            "stdout:\n{}\nstderr: {}", stdout, stderr);
-
-    assert!(stderr.contains("No cert to check checksum from "),
-            "stdout:\n{}\nstderr: {}", stdout, stderr);
-    assert!(stderr.contains("Error: Verification failed"),
-            "stdout:\n{}\nstderr: {}", stdout, stderr);
+    let result = sq.verify_maybe(&[], Verify::Message, &msg_pgp, None);
+    assert!(result.is_err());
+    let err = result.unwrap_err().to_string();
+    assert!(err.contains("missing certificate."), "{}", err);
+    assert!(err.contains("Error: Verification failed") , "{}", err);
 
     // Now we use the certificate store.
-    let mut cmd = sq.command();
-    cmd.arg("--cert-store").arg(&certd)
-        .arg("verify")
-        .arg(&msg_pgp);
-    let output = cmd.output().expect("success");
-    let stdout = String::from_utf8_lossy(&output.stdout);
-    let stderr = String::from_utf8_lossy(&output.stderr);
-    assert!(! output.status.success(),
-            "stdout:\n{}\nstderr: {}", stdout, stderr);
+    let result = sq.verify_maybe(
+        &["--cert-store", &certd], Verify::Message, &msg_pgp, None);
+    assert!(result.is_err());
+    let err = result.unwrap_err().to_string();
 
     // The default trust model says that certificates from the
     // certificate store are not authenticated.
-    assert!(stderr.contains("Unauthenticated checksum from "),
-            "stdout:\n{}\nstderr: {}", stdout, stderr);
-    assert!(stderr.contains("Error: Verification failed"),
-            "stdout:\n{}\nstderr: {}", stdout, stderr);
+    assert!(err.contains("the certificate can't be authenticated."),
+            "{}", err);
+    assert!(err.contains("Error: Verification failed"),
+            "{}", err);
 
     // Now we use the certificate store *and* specify the certificate.
     let mut cmd = sq.command();
     cmd.arg("--cert-store").arg(&certd)
-        .arg("verify")
-        .arg("--signer-cert").arg(&alice.fingerprint().to_string())
+        .arg("verify").arg("--message")
+        .arg("--signer").arg(&alice.fingerprint().to_string())
         .arg(&msg_pgp);
     let output = cmd.output().expect("success");
     let stdout = String::from_utf8_lossy(&output.stdout);
@@ -995,9 +840,9 @@ fn sq_sign_using_cert_store() -> Result<()> {
 
     // The default trust model says that certificates from the
     // certificate store are not authenticated.
-    assert!(stderr.contains("Good signature from "),
+    assert!(stderr.contains("Authenticated signature made by "),
             "stdout:\n{}\nstderr: {}", stdout, stderr);
-    assert!(stderr.contains("1 good signature."),
+    assert!(stderr.contains("1 authenticated signature."),
             "stdout:\n{}\nstderr: {}", stdout, stderr);
 
     Ok(())
@@ -1042,7 +887,7 @@ fn sq_verify_wot() -> Result<()> {
         for trust_root in trust_roots {
             cmd.args(&["--trust-root", trust_root]);
         }
-        cmd.arg("verify");
+        cmd.arg("verify").arg("--message");
         for signer_file in signer_files {
             cmd.args(&["--signer-file", signer_file]);
         }
@@ -1058,19 +903,28 @@ fn sq_verify_wot() -> Result<()> {
     //
     // The certification is imported into the cert store.
     let sq_certify = |sq: &Sq,
-                      key: &str, cert: &str, userid: &str,
+                      certifier: &str, cert: &str, userid: &str,
                       trust_amount: Option<usize>|
     {
-        let mut cmd = sq.command();
-        cmd.args(&["pki", "certify", "--certifier-file", key, cert, userid]);
+        let mut extra_args = Vec::new();
+        let trust_amount_;
         if let Some(trust_amount) = trust_amount {
-            cmd.args(&["--amount", &trust_amount.to_string()[..]]);
+            extra_args.push("--amount");
+            trust_amount_ = format!("{}", trust_amount);
+            extra_args.push(&trust_amount_);
         }
-        let output = sq.run(cmd, true);
 
-        // Import the certification.
-        let certification = sq.base().join("certification");
-        std::fs::write(&certification, &output.stdout).unwrap();
+        let certification = sq.scratch_file(Some(&format!(
+            "certification {} {} by {}", cert, userid, certifier)[..]));
+
+        let cert = if let Ok(kh) = cert.parse::<KeyHandle>() {
+            kh.into()
+        } else {
+            FileOrKeyHandle::FileOrStdin(cert.into())
+        };
+
+        sq.pki_vouch_add(&extra_args, certifier, cert, &[userid],
+                             Some(certification.as_path()));
         sq.cert_import(&certification);
     };
 
@@ -1087,6 +941,7 @@ fn sq_verify_wot() -> Result<()> {
     // Sign a message.
     sq.command()
         .arg("sign")
+        .arg("--message")
         .args(["--signer-file", &bob_pgp])
         .args(["--signer-file", &carol_pgp])
         .args(["--signer-file", &dave_pgp])
@@ -1109,7 +964,7 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[], &[&alice_pgp], &msg_pgp);
         assert!(! output.0.success());
 
-        // But, one good signature is enough.
+        // But, one authenticated signature is enough.
         let output = sq_verify(&sq, &[], &[&alice_pgp, &bob_pgp], &msg_pgp);
         assert!(output.0.success());
     }
@@ -1121,7 +976,7 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[], &[], &msg_pgp);
         assert!(! output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Unauthenticated checksum from "),
+        assert!(output.2.contains("the certificate can't be authenticated."),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
 
@@ -1130,7 +985,7 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[&alice_fpr], &[], &msg_pgp);
         assert!(! output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Unauthenticated checksum from "),
+        assert!(output.2.contains("the certificate can't be authenticated."),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
     }
@@ -1140,7 +995,7 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[&bob_fpr], &[], &msg_pgp);
         assert!(output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Good signature from "),
+        assert!(output.2.contains("Authenticated signature made by "),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
 
@@ -1148,7 +1003,7 @@ fn sq_verify_wot() -> Result<()> {
             &sq, &[&alice_fpr, &bob_fpr], &[], &msg_pgp);
         assert!(output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Good signature from "),
+        assert!(output.2.contains("Authenticated signature made by "),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
     }
@@ -1162,7 +1017,7 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[&alice_fpr], &[], &msg_pgp);
         assert!(! output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Unauthenticated checksum from "),
+        assert!(output.2.contains("the certificate can't be authenticated."),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
     }
@@ -1177,10 +1032,10 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[&alice_fpr], &[], &msg_pgp);
         assert!(! output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Unauthenticated checksum from "),
+        assert!(output.2.contains("the certificate can't be authenticated."),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
-        assert!(output.2.contains("3 unauthenticated checksums"),
+        assert!(output.2.contains("3 unauthenticated signatures"),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
     }
@@ -1194,10 +1049,10 @@ fn sq_verify_wot() -> Result<()> {
         let output = sq_verify(&sq, &[&alice_fpr], &[], &msg_pgp);
         assert!(output.0.success(),
                 "stdout:\n{}\nstderr:\n{}", output.1, output.2);
-        assert!(output.2.contains("Good signature from "),
+        assert!(output.2.contains("Authenticated signature made by "),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
-        assert!(output.2.contains("1 good signature, 2 unauthenticated checksums"),
+        assert!(output.2.contains("1 authenticated signature, 2 unauthenticated signatures"),
                 "stdout:\n{}\nstderr:\n{}",
                 output.1, output.2);
     }
@@ -1218,13 +1073,14 @@ fn sq_sign_keyring() {
     let mut alice_pub = alice_pgp.clone();
     alice_pub.set_extension("pub");
 
-    sq.toolbox_extract_cert(&alice_pgp, Some(&*alice_pub));
+    sq.key_delete(&alice_pgp, Some(&*alice_pub));
 
     // We pass the secret key material via --keyring.  This should
     // work.
     let mut cmd = sq.command();
     cmd.arg("--keyring").arg(&alice_pgp)
         .arg("sign")
+        .arg("--message")
         .arg("--signer-file").arg(&alice_pub);
 
     sq.run(cmd, Some(true));
@@ -1232,7 +1088,33 @@ fn sq_sign_keyring() {
     // If we don't pass the secret key material, this should fail.
     let mut cmd = sq.command();
     cmd.arg("sign")
+        .arg("--message")
         .arg("--signer-file").arg(&alice_pub);
 
     sq.run(cmd, Some(false));
 }
+
+/// Creates a text signature.
+#[test]
+fn sq_sign_mode_text() -> Result<()> {
+    let sq = Sq::new();
+
+    let (_alice, alice_pgp, _alice_rev)
+        = sq.key_generate(&[], &["alice"]);
+
+    let data_n = sq.scratch_dir().join("data_n");
+    std::fs::write(&data_n, "Hello\n")?;
+
+    let sig = sq.scratch_dir().join("sig");
+    sq.sign_detached(&["--mode=text"], alice_pgp.as_path(),
+                     data_n.as_path(), sig.as_path());
+
+    let verify_args = ["--signer-file", &alice_pgp.display().to_string()];
+    sq.verify(&verify_args, Verify::SignatureFile(sig.clone()), &data_n, None);
+
+    let data_rn = sq.scratch_dir().join("data_rn");
+    std::fs::write(&data_rn, "Hello\r\n")?;
+    sq.verify(&verify_args, Verify::SignatureFile(sig.clone()), &data_rn, None);
+
+    Ok(())
+}
